package com.iconplus.smartproc.controller;

import com.iconplus.smartproc.helper.model.EmptyResponse;
import com.iconplus.smartproc.model.request.RksDaftarIsiRequest;
import com.iconplus.smartproc.model.request.RksTemplateRequest;
import com.iconplus.smartproc.model.response.*;
import com.iconplus.smartproc.service.rkstemplate.*;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.web.bind.annotation.*;

@CrossOrigin(origins = "${fe.server}", allowCredentials = "true")
@RestController
@RequestMapping("/api/rks/template")
public class RksTemplateController {

    private final PostCreateRksTemplateService postCreateRksTemplateService;
    private final GetListRksTemplateService getListRksTemplateService;
    private final PutUpdateRksTemplateService putUpdateRksTemplateService;
    private final DeleteRksTemplateService deleteRksTemplateService;
    private GetTemplateListRksDaftarIsiService getTemplateListRksDaftarIsiService;
    public RksTemplateController(PostCreateRksTemplateService postCreateRksTemplateService,
                                 GetListRksTemplateService getListRksTemplateService,
                                 PutUpdateRksTemplateService putUpdateRksTemplateService,
                                 DeleteRksTemplateService deleteRksTemplateService,
                                 GetTemplateListRksDaftarIsiService getTemplateListRksDaftarIsiService) {
        this.postCreateRksTemplateService = postCreateRksTemplateService;
        this.getListRksTemplateService = getListRksTemplateService;
        this.putUpdateRksTemplateService = putUpdateRksTemplateService;
        this.deleteRksTemplateService = deleteRksTemplateService;
        this.getTemplateListRksDaftarIsiService = getTemplateListRksDaftarIsiService;
    }

    @PostMapping()
    public RksTemplateResponse createRksTemplate(@RequestBody RksTemplateRequest rksTemplateRequest) {
        return postCreateRksTemplateService.execute(rksTemplateRequest);
    }

    @GetMapping()
    public ListRksTemplateResponse getListRksTemplate(@RequestParam(name = "page", defaultValue = "1") Integer page,
                                                      @RequestParam(name = "size", defaultValue = "5") Integer size) {

        Pageable pageable = PageRequest.of((page - 1), size);
        RksTemplateRequest rksTemplateRequest = RksTemplateRequest.builder()
                .pageable(pageable)
                .build();

        return getListRksTemplateService.execute(rksTemplateRequest);

    }

    @PutMapping("/{id}")
    public RksTemplateResponse editRksTemplate(@PathVariable(name = "id") Long id,
                                               @RequestBody RksTemplateRequest rksTemplateRequest) {
        rksTemplateRequest.setId(id);
        return putUpdateRksTemplateService.execute(rksTemplateRequest);
    }

    @DeleteMapping("/{id}")
    public EmptyResponse deleteRksTemplate(@PathVariable(name = "id") Long id) {
        return deleteRksTemplateService.execute(RksTemplateRequest.builder()
                .id(id)
                .build());

    }


    @GetMapping("/{id}")
    public ListRksDaftarIsiResponse getRksDaftarIsiResponse(@PathVariable(name = "id") Long id,
                                                            @RequestParam(name = "page", defaultValue = "1") Integer page,
                                                            @RequestParam(name = "size", defaultValue = "5") Integer size) {

        Pageable pageable = PageRequest.of((page - 1), size);
        RksTemplateRequest rksTemplateRequest = RksTemplateRequest.builder()
                .id(id)
                .pageable(pageable)
                .build();

        return getTemplateListRksDaftarIsiService.execute(rksTemplateRequest);
    }
}
